@@ -426,21 +426,32 @@ int Interface::setOptionsMap(const QMap<QString, QString> &options)
 
     Option *sourceOption = getOption(SourceOption);
     Option *modeOption = getOption(ScanModeOption);
+    Option *resolutionOption = getOption(ResolutionOption);
 
     // Priorize source option
-    if (sourceOption != nullptr && optionMapCopy.contains(sourceOption->name())) {
-        if (sourceOption->setValue(optionMapCopy[sourceOption->name()])) {
+    auto it = optionMapCopy.find(sourceOption->name());
+    if (sourceOption != nullptr && it != optionMapCopy.end()) {
+        if (sourceOption->setValue(it.value())) {
             ret++;
         }
-        optionMapCopy.remove(sourceOption->name());
+        optionMapCopy.erase(it);
     }
 
     // Priorize mode option
-    if (modeOption != nullptr && optionMapCopy.contains(modeOption->name())) {
-        if (modeOption->setValue(optionMapCopy[modeOption->name()])) {
+    it = optionMapCopy.find(modeOption->name());
+    if (modeOption != nullptr && it != optionMapCopy.end()) {
+        if (modeOption->setValue(it.value())) {
             ret++;
         }
-        optionMapCopy.remove(modeOption->name());
+        optionMapCopy.erase(it);
+    }
+
+    // Get iterator to resolution option, but do not apply value
+    it = optionMapCopy.find(resolutionOption->name());
+    QString value;
+    if (resolutionOption != nullptr && it != optionMapCopy.end()) {
+        value = it.value();
+        optionMapCopy.erase(it);
     }
 
     // Update remaining options
@@ -450,6 +461,13 @@ int Interface::setOptionsMap(const QMap<QString, QString> &options)
             ret++;
         }
     }
+
+    // Apply resolution value at the latest, as this option is likely
+    // reset to a default value by SANE when setting other options
+    if (resolutionOption->setValue(value)) {
+        ret++;
+    }
+
     return ret;
 }
 
